@@ -28,7 +28,6 @@
  *   NV-CONTROL -> event -> glib -> CtkEvent -> signal -> GUI
  */
 
-
 #include <string.h>
 
 #include <gtk/gtk.h>
@@ -40,7 +39,7 @@
 #include "NVCtrlLib.h"
 #include "msg.h"
 
-static void ctk_event_class_init(CtkEventClass *ctk_event_class);
+static void ctk_event_class_init(CtkEventClass *ctk_event_class, gpointer);
 
 static gboolean ctk_event_prepare(GSource *, gint *);
 static gboolean ctk_event_check(GSource *);
@@ -102,7 +101,8 @@ GType ctk_event_get_type(void)
 } /* ctk_event_get_type() */
 
 
-static void ctk_event_class_init(CtkEventClass *ctk_event_class)
+static void ctk_event_class_init(CtkEventClass *ctk_event_class,
+                                 gpointer class_data)
 {
     gint i;
 
@@ -243,7 +243,6 @@ static void ctk_event_class_init(CtkEventClass *ctk_event_class)
     MAKE_SIGNAL(NV_CTRL_THERMAL_SENSOR_READING);
     MAKE_SIGNAL(NV_CTRL_THERMAL_SENSOR_PROVIDER);
     MAKE_SIGNAL(NV_CTRL_THERMAL_SENSOR_TARGET);
-    MAKE_SIGNAL(NV_CTRL_GPU_CURRENT_PROCESSOR_CLOCK_FREQS);
     MAKE_SIGNAL(NV_CTRL_GPU_PCIE_MAX_LINK_SPEED);
     MAKE_SIGNAL(NV_CTRL_3D_VISION_PRO_TRANSCEIVER_CHANNEL);
     MAKE_SIGNAL(NV_CTRL_3D_VISION_PRO_TRANSCEIVER_MODE);
@@ -263,8 +262,6 @@ static void ctk_event_class_init(CtkEventClass *ctk_event_class)
     MAKE_SIGNAL(NV_CTRL_FRAMELOCK_DISPLAY_CONFIG);
     MAKE_SIGNAL(NV_CTRL_TOTAL_DEDICATED_GPU_MEMORY);
     MAKE_SIGNAL(NV_CTRL_USED_DEDICATED_GPU_MEMORY);
-    MAKE_SIGNAL(NV_CTRL_GPU_DOUBLE_PRECISION_BOOST_IMMEDIATE);
-    MAKE_SIGNAL(NV_CTRL_GPU_DOUBLE_PRECISION_BOOST_REBOOT);
     MAKE_SIGNAL(NV_CTRL_DPY_HDMI_3D);
     MAKE_SIGNAL(NV_CTRL_BASE_MOSAIC);
     MAKE_SIGNAL(NV_CTRL_MULTIGPU_PRIMARY_POSSIBLE);
@@ -297,6 +294,10 @@ static void ctk_event_class_init(CtkEventClass *ctk_event_class)
     MAKE_SIGNAL(NV_CTRL_DISPLAY_VRR_MIN_REFRESH_RATE);
     MAKE_SIGNAL(NV_CTRL_DISPLAY_VRR_ENABLED);
     MAKE_SIGNAL(NV_CTRL_PLATFORM_POWER_MODE);
+    MAKE_SIGNAL(NV_CTRL_MUX_AUTO_SWITCH);
+    MAKE_SIGNAL(NV_CTRL_FRAMELOCK_MULTIPLY_DIVIDE_VALUE);
+    MAKE_SIGNAL(NV_CTRL_FRAMELOCK_MULTIPLY_DIVIDE_MODE);
+    MAKE_SIGNAL(NV_CTRL_NUMBER_OF_HARDWARE_HEADS_USED);
 #undef MAKE_SIGNAL
 
     /*
@@ -306,7 +307,7 @@ static void ctk_event_class_init(CtkEventClass *ctk_event_class)
      * knows about.
      */
 
-#if NV_CTRL_LAST_ATTRIBUTE != NV_CTRL_PLATFORM_CURRENT_POWER_MODE
+#if NV_CTRL_LAST_ATTRIBUTE != NV_CTRL_NUMBER_OF_HARDWARE_HEADS_USED
 #warning "There are attributes that do not emit signals!"
 #endif
 
@@ -351,9 +352,10 @@ static void ctk_event_class_init(CtkEventClass *ctk_event_class)
     MAKE_STRING_SIGNAL(NV_CTRL_STRING_GPU_UTILIZATION);
     MAKE_STRING_SIGNAL(NV_CTRL_STRING_MULTIGPU_MODE);
     MAKE_STRING_SIGNAL(NV_CTRL_STRING_PRIME_OUTPUTS_DATA);
+    MAKE_STRING_SIGNAL(NV_CTRL_STRING_MUX_STATE);
 #undef MAKE_STRING_SIGNAL
 
-#if NV_CTRL_STRING_LAST_ATTRIBUTE != NV_CTRL_STRING_DISPLAY_NAME_CONNECTOR
+#if NV_CTRL_STRING_LAST_ATTRIBUTE != NV_CTRL_STRING_MUX_STATE
 #warning "There are attributes that do not emit signals!"
 #endif
 
@@ -386,7 +388,8 @@ static void ctk_event_class_init(CtkEventClass *ctk_event_class)
     MAKE_BINARY_SIGNAL(NV_CTRL_BINARY_DATA_DISPLAYS_ON_GPU);
 #undef MAKE_BINARY_SIGNAL
 
-#if NV_CTRL_BINARY_DATA_LAST_ATTRIBUTE != NV_CTRL_BINARY_DATA_DISPLAYS_ON_GPU
+#if NV_CTRL_BINARY_DATA_LAST_ATTRIBUTE != \
+    NV_CTRL_BINARY_DATA_GPU_ECC_DETAILED_ERRORS_DOUBLE_BIT_AGGREGATE
 #warning "There are attributes that do not emit signals!"
 #endif
 
